<!-- Unused files
 
givenDebugOutputVerbose.json
givenLongOutput.json
 

Link file with relative path: <a href="./_media/examples/zencode_cookbook/givenArraysLoadInput.json">givenArraysLoadInput.json</a>
 
-->




# W3C Verifiable Credentials scenario

This scenario implements signature and verification according to the  [W3C Verifiable Credentials standard](https://w3c.github.io/vc-data-model/).

The scripts focus on embedding a signature into a JWS string as specified in [rfc7797](https://tools.ietf.org/html/rfc7797), the implementation of this can be found in [zencode_w3c.lua](https://github.com/dyne/Zenroom/blob/master/src/lua/zencode_w3c.lua#L50-L61)

## Signature


The data we used to sign is a standard one that can be found on the [W3C VC page](https://w3c.github.io/vc-data-model/#concrete-lifecycle-example), to which we added a string named *pubkey_url* containing the *verificationMethod*, but any JSON would work: 

[](../_media/examples/zencode_cookbook/W3C-VC_unsigned.json ':include :type=code json')

The script will: 
* Sign thes JSON, currently using *secp256k1* (which may changed if needed)
* Format the signature as JWS
* Insert the signature in the JSON in the *proof* object
* Insert strings *proofPurpose* and *type* in the JSON
* Insert the *verificationMethod* in the JSON

The script looks like:

[](../_media/examples/zencode_cookbook/W3C-VC_sign.zen ':include :type=code gherkin')

And it also requires an ECDH keypair: 

[](../_media/examples/zencode_cookbook/W3C-VC_issuerKeypair.json ':include :type=code json')

The output should look like: 

[](../_media/examples/zencode_cookbook/W3C-VC_signed.json ':include :type=code json') 

## Verification
 
The script verifies if the signature matches the public key passed to it, which needs to be retrieved from the *verificationMethod* in advance (also possible using [Restroom-mw] (https://dyne.github.io/restroom-mw/#/)).

The input expected, apart from the signed JSON, is: 

[](../_media/examples/zencode_cookbook/W3C-VC_pubkey.json ':include :type=code json')

and the script looks like: 


[](../_media/examples/zencode_cookbook/W3C-VC_verify.zen ':include :type=code gherkin')

The script will simply output the text "**W3C CREDENTIAL IS VALID**" , but it can be used as a condition to execute further statements. 


# Extract Verification Method

If you want to extract the "*verificationMethod*" string from a signed credential using Zenroom, you can use the script: 


[](../_media/examples/zencode_cookbook/W3C-VC_extract.zen ':include :type=code gherkin')

Which should output: 

[](../_media/examples/zencode_cookbook/W3C-VC_extracted_verification_method.json ':include :type=code json')


# The script used to create the material in this page

All the smart contracts and the data you see in this page are generated by the scripts [run-when.sh](https://github.com/dyne/Zenroom/blob/master/test/zencode_w3c/run.sh). If you want to run the scripts (on Linux) you should: 
 - *git clone https://github.com/dyne/Zenroom.git*
 - install **zsh** and **jq**
 - download a [zenroom binary](https://zenroom.org/#downloads) and place it */bin* or */usr/bin* or in *./Zenroom/src*









<!-- Temp removed, 


-->
### 
