
# Branching: If/Endif statements

Zenroom allows branching, done using the ***If*** and ***Endif*** Statements. 

The **first** ***If*** statement creates a *conditional branch* and can be *stacked*: you can have multiple conditions following each other, something like: 

* If (condition1)
  * If (condition2)
    * If (condition3)
		* When (do something)
		* Then (print something)
* Endif

The statement ***Endif*** ends the conditional branch, so takes the execution back to the *main* execution branch, indepently on the amount of conditions (on how many ***If*** you have stacked together).

You can not use a ***When*** or ***Then*** statement, inside a stack of ***If***, so something like this will return an error: 

* If (condition1)
* <span style="color: red">When (do something)</span>
  * If (condition2)
  * <span style="color: red">Then (print something)</span>
    * If (condition3)
	* Then (print something)
* Endif



## Simple If/Endif example 

We'll use a simple JSON file with two numbers as input:

[](../_media/examples/zencode_cookbook/branching/leftrightA.json ':include :type=code json')

And in this script we can see: 
* a simple condition, that will fail 
* two stacked conditions, where the second one will fail  
* a simple condition, that will succeed
* a simple condition, based on output from a the previous branch, that will succeed

The script goes: 

[](../_media/examples/zencode_cookbook/branching/branchA.zen ':include :type=code gherkin')

You should expect an output like this: 

[](../_media/examples/zencode_cookbook/branching/outputA.json ':include :type=code json')



## Statements that can be used as a condition 

Only some statements can be used as a condition, typically the ones that **verify** a value or check do cryptographic checks, specifically:

* all the **verify** statements
* all the **is less than** and **is more than** statements
* all the **is equal to** and **is not equal to** statements
* all the **is found** and **is not found** statements



# The script used to create the material in this page

All the smart contracts and the data you see in this page are generated by the scripts [run.sh](https://github.com/dyne/Zenroom/blob/master/test/zencode_branching/run.sh). If you want to run the scripts (on Linux) you should: 
 - *git clone https://github.com/dyne/Zenroom.git*
 - install **zsh** and **jq**
 - download a [zenroom binary](https://zenroom.org/#downloads) and place it */bin* or */usr/bin* or in *./Zenroom/src*



<!-- Temp removed, 


-->